/*
   Copyright (c) 1995-1996 by Cisco systems, Inc.

   Permission to use, copy, modify, and distribute this software for
   any purpose and without fee is hereby granted, provided that this
   copyright and permission notice appear on all copies of the
   software and supporting documentation, the name of Cisco Systems,
   Inc. not be used in advertising or publicity pertaining to
   distribution of the program without specific prior permission, and
   notice be given in supporting documentation that modification,
   copying and distribution is by permission of Cisco Systems, Inc.

   Cisco Systems, Inc. makes no representations about the suitability
   of this software for any purpose.  THIS SOFTWARE IS PROVIDED ``AS
   IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
   WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "tac_plus.h"
#include <fcntl.h>

#if defined (ALIASES) || defined(TTYS)
 #include "own_wtmp.h"
#endif

static int acctfd = 0;

/* Make a acct entry into the accounting file for accounting.
   Return 1 on error  */

static int
acct_write(string)
    char *string;
{
    if (write(acctfd, string, strlen(string)) != strlen(string)) {
	report(LOG_ERR, "%s: couldn't write acct file %s %s",
	       session.peer,
	       session.acctfile, strerror(errno));
	return(1);
    }

    if (debug & DEBUG_ACCT_FLAG)
	report(LOG_DEBUG, "'%s'", string);

    return(0);
}

/* Write a string or "unknown" into the accounting file.
   Return 1 on error  */
static int
acct_write_field(string)
    char *string;
{
    if (string && string[0]) {
	if (acct_write(string))
	    return(1);
    } else {
	if (acct_write("unknown"))
	    return(1);
    }
    return(0);
}

int
do_acct(rec)
struct acct_rec *rec;
{
    int i, errors;
    time_t t = time(NULL);

#ifndef NO_BWM_DATE
    char atime[256];
#endif

#ifdef ALIASES
    char *s;
#endif

#ifdef UTMP
    int goodname = (rec->identity->username && *rec->identity->username != '\0');
    if(
       (rec->authen_service == TAC_PLUS_AUTHEN_SVC_LOGIN ||
        rec->authen_service == TAC_PLUS_AUTHEN_SVC_PPP) &&
        (rec->acct_type == ACCT_TYPE_START || rec->acct_type == ACCT_TYPE_STOP)
#ifndef LOG_EMPTY
        && goodname
#endif
      )
     {
      logintac(
               (rec->acct_type == ACCT_TYPE_START) ?
#ifdef LOG_EMPTY
                      ((goodname) ? rec->identity->username : EMPTY_NAME)
#else
                      rec->identity->username
#endif
                                                   : "",
               t,
               rec->identity->NAS_name,
#ifdef ALIASES
               rec->identity->NAS_alias,
#else
               rec->identity->NAS_name,
#endif
               rec->identity->NAS_port,
               rec->identity->NAS_palias,
               rec->identity->NAS_slot,
               rec->identity->NAS_flags
              );
     }
#endif

#if defined(ALIASES) || defined(TTYS)
    if(!(rec->identity->NAS_flags & A_FLAG_ACCT_X))
      return 0;
#endif

    if (!acctfd) {
	acctfd = open(session.acctfile, O_CREAT | O_WRONLY | O_APPEND, 0644);
	if (acctfd < 0) {
	    report(LOG_ERR, "Can't open acct file %s -- %s",
		   session.acctfile, strerror(errno));
	    return(1);
	}
    }
    if (!tac_lockfd(session.acctfile, acctfd)) {
	rec->admin_msg = tac_strdup("Cannot lock log file");
	report(LOG_ERR, "%s: Cannot lock %s",
	       session.peer, session.acctfile);
	return(1);
    }

    errors = 0;

#ifdef NO_BWM_DATE
    {
     char *ct = ctime(&t);
     ct[24] = '\0';
     errors += acct_write(ct);
    }
#else
    strftime(atime, sizeof(atime), "%Y/%m/%d %H:%M:%S", localtime(&t));
    errors += acct_write(atime);
#endif

    errors += acct_write("\t");

#ifndef NO_BWM_DATE
    sprintf(atime, "%u", t);
    errors += acct_write(atime);
    errors += acct_write("\t");
#endif

#ifdef ALIASES
    switch(rec->identity->NAS_flags & A_FLAG_ACCT_X)
     {
          case A_FLAG_ACCT_TR:
            s = rec->identity->NAS_name;
            break;
          case A_FLAG_ACCT_AL:
            s = rec->identity->NAS_alias;
            break;
          default:
            s = (rec->identity->NAS_flags & A_FLAG_FOUND_AL) ?
                        rec->identity->NAS_alias : rec->identity->NAS_name;
            break;

     }
    errors += acct_write_field(s);
#else
    errors += acct_write_field(rec->identity->NAS_name);
#endif
    errors += acct_write("\t");

    errors += acct_write_field(rec->identity->username);
    errors += acct_write("\t");

#ifdef TTYS
    errors += acct_write_field((rec->identity->NAS_flags & A_FLAG_PACCT_AL) ?
                          rec->identity->NAS_palias : rec->identity->NAS_port);
#else
    errors += acct_write_field(rec->identity->NAS_port);
#endif
    errors += acct_write("\t");

    errors += acct_write_field(rec->identity->NAC_address);
    errors += acct_write("\t");

    switch(rec->acct_type) {
    case ACCT_TYPE_UPDATE:
	errors += acct_write("update\t");
	break;
    case ACCT_TYPE_START:
	errors += acct_write("start\t");
	break;
    case ACCT_TYPE_STOP:
	errors += acct_write("stop\t");
	break;
    default:
	errors += acct_write("unknown\t");
	break;
    }

    for (i=0; i < rec->num_args; i++) {
	errors += acct_write(rec->args[i]);
	if (i < (rec->num_args-1))
	    errors += acct_write("\t");
    }
/* **** pasa's debug code
     sprintf(atime, "\txmethod=%d\txtype=%d\txservice=%d",
              rec->authen_method, rec->authen_type, rec->authen_service);
     errors += acct_write(atime);
 **** */
    errors += acct_write("\n");

    close(acctfd);
    acctfd = 0;

    if (errors) {
	return(1);
    }
    return (0);
}

